@@ -6,9 +6,9 @@
 /*                                                                          */
 /*                              C Header File                               */
 /*                                                                          */
-/*                            $Revision: 1.50 $                             */
+/*                            $Revision: 1.54 $                             */
 /*                                                                          */
-/*          Copyright (C) 1992-1997, Free Software Foundation, Inc.         */
+/*          Copyright (C) 1992-1998, Free Software Foundation, Inc.         */
 /*                                                                          */
 /* GNAT is free software;  you can  redistribute it  and/or modify it under */
 /* terms of the  GNU General Public License as published  by the Free Soft- */
@@ -280,10 +280,11 @@ Present (N)
   return N != Empty;
 }
 
-/* Test the range of N to distinguish between the cases of Node_Id, List_Id
-   and Elist_Id arguments.  */
 extern Node_Id Parent		PROTO((Tree_Id));
 
+#define Current_Error_Node atree__current_error_node
+extern Node_Id Current_Error_Node;
+
 /* Node Access Functions:  */
 
 #define Nkind(N)        ((Node_Kind)(Nodes_Ptr [N].U.K.kind))
@@ -316,64 +317,68 @@ extern Node_Id Parent		PROTO((Tree_Id));
 #define Field22(N)    (Nodes_Ptr [N+3].V.EX.field9)
 #define Field23(N)    (Nodes_Ptr [N+3].V.EX.field10)
 
-#define Node1(N)      (Nodes_Ptr [N].V.NX.field1)
-#define Node2(N)      (Nodes_Ptr [N].V.NX.field2)
-#define Node3(N)      (Nodes_Ptr [N].V.NX.field3)
-#define Node4(N)      (Nodes_Ptr [N].V.NX.field4)
-#define Node5(N)      (Nodes_Ptr [N].V.NX.field5)
-#define Node6(N)      (Nodes_Ptr [N+1].V.EX.field6)
-#define Node7(N)      (Nodes_Ptr [N+1].V.EX.field7)
-#define Node8(N)      (Nodes_Ptr [N+1].V.EX.field8)
-#define Node9(N)      (Nodes_Ptr [N+1].V.EX.field9)
-#define Node10(N)     (Nodes_Ptr [N+1].V.EX.field10)
-#define Node11(N)     (Nodes_Ptr [N+1].V.EX.X.field11)
-#define Node12(N)     (Nodes_Ptr [N+1].V.EX.U.field12)
-#define Node13(N)     (Nodes_Ptr [N+2].V.EX.field6)
-#define Node14(N)     (Nodes_Ptr [N+2].V.EX.field7)
-#define Node15(N)     (Nodes_Ptr [N+2].V.EX.field8)
-#define Node16(N)     (Nodes_Ptr [N+2].V.EX.field9)
-#define Node17(N)     (Nodes_Ptr [N+2].V.EX.field10)
-#define Node18(N)     (Nodes_Ptr [N+2].V.EX.X.field11)
-#define Node19(N)     (Nodes_Ptr [N+3].V.EX.field6)
-#define Node20(N)     (Nodes_Ptr [N+3].V.EX.field7)
-#define Node21(N)     (Nodes_Ptr [N+3].V.EX.field8)
-#define Node22(N)     (Nodes_Ptr [N+3].V.EX.field9)
-#define Node23(N)     (Nodes_Ptr [N+3].V.EX.field10)
+#define Node1(N)      Field1 (N)
+#define Node2(N)      Field2 (N)
+#define Node3(N)      Field3 (N)
+#define Node4(N)      Field4 (N)
+#define Node5(N)      Field5 (N)
+#define Node6(N)      Field6 (N)
+#define Node7(N)      Field7 (N)
+#define Node8(N)      Field8 (N)
+#define Node9(N)      Field9 (N)
+#define Node10(N)     Field10 (N)
+#define Node11(N)     Field11 (N)
+#define Node12(N)     Field12 (N)
+#define Node13(N)     Field13 (N)
+#define Node14(N)     Field14 (N)
+#define Node15(N)     Field15 (N)
+#define Node16(N)     Field16 (N)
+#define Node17(N)     Field17 (N)
+#define Node18(N)     Field18 (N)
+#define Node19(N)     Field19 (N)
+#define Node20(N)     Field20 (N)
+#define Node21(N)     Field21 (N)
+#define Node22(N)     Field22 (N)
+#define Node23(N)     Field23 (N)
 
-#define List1(N)      (Nodes_Ptr [N].V.NX.field1)
-#define List2(N)      (Nodes_Ptr [N].V.NX.field2)
-#define List3(N)      (Nodes_Ptr [N].V.NX.field3)
-#define List4(N)      (Nodes_Ptr [N].V.NX.field4)
-#define List5(N)      (Nodes_Ptr [N].V.NX.field5)
+#define List1(N)      Field1 (N)
+#define List2(N)      Field2 (N)
+#define List3(N)      Field3 (N)
+#define List4(N)      Field4 (N)
+#define List5(N)      Field5 (N)
+#define List10(N)     Field10 (N)
+#define List23(N)     Field23 (N)
 
-#define Elist3(N)     (Nodes_Ptr [N].V.NX.field3)
-#define Elist6(N)     (Nodes_Ptr [N+1].V.EX.field6)
-#define Elist7(N)     (Nodes_Ptr [N+1].V.EX.field7)
-#define Elist13(N)    (Nodes_Ptr [N+2].V.EX.field6)
-#define Elist14(N)    (Nodes_Ptr [N+2].V.EX.field7)
-#define Elist19(N)    (Nodes_Ptr [N+3].V.EX.field6)
+#define Elist3(N)     Field3 (N)
+#define Elist4(N)     Field4 (N)
+#define Elist8(N)     Field8 (N)
+#define Elist13(N)    Field13 (N)
+#define Elist14(N)    Field14 (N)
+#define Elist18(N)    Field18 (N)
+#define Elist21(N)    Field21 (N)
 
-#define Name1(N)      (Nodes_Ptr [N].V.NX.field1)
-#define Name2(N)      (Nodes_Ptr [N].V.NX.field2)
+#define Name1(N)      Field1 (N)
+#define Name2(N)      Field2 (N)
 
-#define Str3(N)       (Nodes_Ptr [N].V.NX.field3)
+#define Char_Code2(N) (Field2 (N) - Char_Code_Bias)
 
-#define Char_Code2(N) (Nodes_Ptr [N].V.NX.field2 - Char_Code_Bias)
+#define Str3(N)       Field3 (N)
 
-#define Uint3(N)      (Nodes_Ptr [N].V.NX.field3)
-#define Uint4(N)      (Nodes_Ptr [N].V.NX.field4)
-#define Uint8(N)      (Nodes_Ptr [N+1].V.EX.field8)
-#define Uint9(N)      (Nodes_Ptr [N+1].V.EX.field9)
-#define Uint11(N)     (Nodes_Ptr [N+1].V.EX.X.field11)
-#define Uint12(N)     (Nodes_Ptr [N+1].V.EX.U.field12)
-#define Uint13(N)     (Nodes_Ptr [N+2].V.EX.field6)
-#define Uint15(N)     (Nodes_Ptr [N+2].V.EX.field8)
-#define Uint16(N)     (Nodes_Ptr [N+2].V.EX.field9)
-#define Uint22(N)     (Nodes_Ptr [N+3].V.EX.field9)
+#define Uint3(N)      Field3 (N)
+#define Uint4(N)      Field4 (N)
+#define Uint8(N)      Field8 (N)
+#define Uint9(N)      Field9 (N)
+#define Uint11(N)     Field11 (N)
+#define Uint12(N)     Field12 (N)
+#define Uint13(N)     Field13 (N)
+#define Uint15(N)     Field15 (N)
+#define Uint16(N)     Field16 (N)
+#define Uint17(N)     Field17 (N)
+#define Uint22(N)     Field22 (N)
 
-#define Ureal3(N)     (Nodes_Ptr [N].V.NX.field3)
-#define Ureal6(N)     (Nodes_Ptr [N+1].V.EX.field6)
-#define Ureal7(N)     (Nodes_Ptr [N+1].V.EX.field7)
+#define Ureal3(N)     Field3 (N)
+#define Ureal18(N)    Field18 (N)
+#define Ureal21(N)    Field21 (N)
 
 #define Analyzed(N)          (Nodes_Ptr [N].U.K.analyzed)
 #define Comes_From_Source(N) (Nodes_Ptr [N].U.K.comes_from_source)
