@@ -6,9 +6,9 @@
 /*                                                                          */
 /*                                  Body                                    */
 /*                                                                          */
-/*                             $Revision: 1.8 $                             */
+/*                             $Revision: 1.15 $                            */
 /*                                                                          */
-/*          Copyright (C) 1992-1996 Free Software Foundation, Inc.          */
+/*          Copyright (C) 1992-1999 Free Software Foundation, Inc.          */
 /*                                                                          */
 /* GNAT is free software;  you can  redistribute it  and/or modify it under */
 /* terms of the  GNU General Public License as published  by the Free Soft- */
@@ -73,11 +73,9 @@
 #define LONG_DOUBLE_TYPE_SIZE (BITS_PER_WORD * 2)
 #endif
 
-/* The choice of SIZE_TYPE here is very problematic.  We need a signed
-   type whose bit width is Pmode.  Assume "long" is such a type here.  */
-#undef SIZE_TYPE
-#define SIZE_TYPE "long int"
-
+#ifndef WIDEST_HARDWARE_FP_SIZE
+#define WIDEST_HARDWARE_FP_SIZE LONG_DOUBLE_TYPE_SIZE
+#endif
 
 /* The following provide a functional interface for the front end Ada code
    to determine the sizes that are used for various C types. */
@@ -139,7 +137,7 @@ get_target_double_size ()
 Pos
 get_target_long_double_size ()
 {
-  return LONG_DOUBLE_TYPE_SIZE;
+  return WIDEST_HARDWARE_FP_SIZE;
 }
 
 Pos
@@ -162,3 +160,28 @@ Nat get_float_words_be () {return FLOAT_WORDS_BIG_ENDIAN;}
 Nat get_words_be       () {return WORDS_BIG_ENDIAN;}
 Nat get_bytes_be       () {return BYTES_BIG_ENDIAN;}
 Nat get_bits_be        () {return BITS_BIG_ENDIAN;}
+Nat get_strict_alignment () {return STRICT_ALIGNMENT;}
+
+Pos
+get_max_priority ()
+{
+#ifdef VXWORKS_TARGET
+  return 200;
+#elif defined(__alpha__) && defined(__osf__)
+  return 60;
+#else
+  return 30;
+#endif
+}
+
+Pos
+get_max_interrupt_priority ()
+{
+#ifdef VXWORKS_TARGET
+  return 255;
+#elif defined(__alpha__) && defined(__osf__)
+  return 63;
+#else
+  return 31;
+#endif
+}
