@@ -4,11 +4,11 @@
 /*                                                                          */
 /*                               A - M I S C                                */
 /*                                                                          */
-/*                          C Implementation File                           */
+/*                           C Implementation File                          */
 /*                                                                          */
-/*                             $Revision: 1.127 $                           */
+/*                             $Revision: 1.153 $
 /*                                                                          */
-/*          Copyright (C) 1992-1997 Free Software Foundation, Inc.          */
+/*          Copyright (C) 1992-1999 Free Software Foundation, Inc.          */
 /*                                                                          */
 /* GNAT is free software;  you can  redistribute it  and/or modify it under */
 /* terms of the  GNU General Public License as published  by the Free Soft- */
@@ -32,9 +32,12 @@
 /*                                                                          */
 /****************************************************************************/
 
+/* This unit needs a global comment saying what is in it ??? */
+
 #include "config.h"
 #include <stdio.h>
 #include <string.h>
+#include <stdarg.h>
 #include "tree.h"
 #include "rtl.h"
 #include "expr.h"
@@ -61,6 +64,7 @@
 
 extern char *xmalloc ();
 extern char *main_input_filename;
+extern FILE *asm_out_file;
 
 /* Tables describing GCC tree codes used only by GNAT.  
 
@@ -109,11 +113,19 @@ char *language_string = "GNU Ada";
 int current_function_returns_null;
 int flag_traditional;		/* Used by dwarfout.c.  */
 
+static void fatal_function	PROTO((char *, va_list));
+static rtx gnat_expand_expr	PROTO((tree, rtx, enum machine_mode,
+				       enum expand_modifier));
+static char *convert_ada_name_to_qualified_name PROTO((char *));
+
 /* Routines Expected by gcc:  */
 
 /* For most front-ends, this is the parser for the language.  For us, we
    process the GNAT tree.  */
 
+#define Set_Jmpbuf_Address system__soft_links__set_jmpbuf_address_soft
+extern void Set_Jmpbuf_Address (void *);
+
 int
 yyparse ()
 {
@@ -128,15 +140,23 @@ yyparse ()
 
   /* Set up to catch unhandled exceptions.  */
   if (__builtin_setjmp (jmpbuf))
-    abort ();
+    {
+      Set_Jmpbuf_Address (0);
+      abort ();
+    }
 
-  system__task_specific_data__set_jmpbuf_address (jmpbuf);
+  /* This is only really needed in longjmp/setjmp mode exceptions
+     but we don't know any easy way to tell what mode the host is
+     compiled in, and it is harmless to do it unconditionally */
+
+  Set_Jmpbuf_Address (jmpbuf);
 
   immediate_size_expand = 1;
 
   /* Call the front end */
   _ada_gnat1drv ();
 
+  Set_Jmpbuf_Address (0);
   return 0;
 }
 
@@ -149,7 +169,7 @@ init_gnat_args ()
   extern char **save_argv;
 
   /* initialize gnat_argv with save_argv size */
-  gnat_argv = (char **) malloc ((save_argc + 1) * sizeof (gnat_argv[0])); 
+  gnat_argv = (char **) xmalloc ((save_argc + 1) * sizeof (gnat_argv[0])); 
 
   gnat_argv [0] = save_argv[0];     /* name of the command */ 
 
@@ -184,7 +204,7 @@ lang_decode_option (p)
     {
       /* recopy the switches without the 'gnat' prefix */
 
-      gnat_argv[gnat_argc] =  (char *) malloc (strlen (p) - 3);
+      gnat_argv[gnat_argc] =  (char *) xmalloc (strlen (p) - 3);
       gnat_argv[gnat_argc][0] = '-';
       strcpy (gnat_argv[gnat_argc] + 1, p + 5);
       gnat_argc ++;
@@ -207,13 +227,58 @@ lang_decode_option (p)
   return 0;
 }
 
+/* Here we have the function to handle the fatal error processing in GCC 2.9.0.
+   Do this only if we have that GCC and VPRINTF is available.  */
+
+#if defined(gen_rtx_PLUS) && defined(HAVE_VPRINTF)
+#define DO_FATAL_FUNCTION
+
+static void
+fatal_function (msgid, ap)
+     char *msgid;
+     va_list ap;
+{
+  char buffer[200];		/* Assume this is big enough.  */
+  char *p, *last_sep = 0;
+  struct template {int first, last;} temp;
+  struct fat_pointer { char *array; struct template *temp; } fp;
+
+  extern comperr__compiler_abort PROTO((struct fat_pointer, int))
+    __attribute__ ((noreturn));
+
+  vsprintf (buffer, msgid, ap);
+
+  /* Find the last directory separator, if any, and start after it.  */
+  for (p = buffer; *p != 0; p++)
+    if (*p == '/'
+#ifdef DIR_SEPARATOR
+	|| *p == DIR_SEPARATOR
+#endif
+	)
+      last_sep = p + 1;
+
+  p = last_sep != 0 ? last_sep : buffer;
+
+  /* Remove a trailing newline, if any.  */
+  if (p[strlen (p) - 1] == '\n')
+    p[strlen (p) - 1] = 0;
+
+  temp.first = 1, temp.last = strlen (p);
+  fp.array = p, fp.temp = &temp;
+
+  Current_Error_Node = error_gnat_node;
+  comperr__compiler_abort (fp, -1);
+}
+#endif
+
 /* Perform all the initialization steps that are language-specific.  */
 
 void
 lang_init ()
 {
 
-  if (!gnat_argc) init_gnat_args ();
+  if (!gnat_argc)
+    init_gnat_args ();
 
   gnat_argv [gnat_argc] = input_filename;   /* name of the main source */
   gnat_argc++;
@@ -221,6 +286,13 @@ lang_init ()
 
   main_input_filename = input_filename;
 
+#ifdef DO_FATAL_FUNCTION
+  set_fatal_function (fatal_function);
+#endif
+
+#if defined(SGI_TARGET) && defined(DWARF2_DEBUGGING_INFO)
+  dwarf2out_set_demangle_name_func (convert_ada_name_to_qualified_name);
+#endif
 }
 
 /* Perform all the finalization steps that are language-specific.  */
@@ -270,7 +342,19 @@ print_lang_decl (file, node, indent)
      FILE *file;
      tree node;
      int indent;
-{}
+{
+  switch (TREE_CODE (node))
+    {
+    case CONST_DECL:
+      print_node (file, "const_corresponding_var",
+		  DECL_CONST_CORRESPONDING_VAR (node), indent + 4);
+      break;
+
+    case FIELD_DECL:
+      print_node (file, "original field", DECL_ORIGINAL_FIELD (node),
+		  indent + 4);
+    }
+}
 
 void
 print_lang_type (file, node, indent)
@@ -313,6 +397,11 @@ print_lang_type (file, node, indent)
       else
 	print_node (file, "Ada size", TYPE_ADA_SIZE (node), indent + 4);
       break;
+
+    case UNION_TYPE:
+    case QUAL_UNION_TYPE:
+      print_node (file, "Ada size", TYPE_ADA_SIZE (node), indent + 4);
+      break;
     }
 }
 
@@ -324,7 +413,7 @@ print_lang_identifier (file, node, indent)
 {}
 
 /* Expands GNAT-specific GCC tree nodes.  The only ones we support here are
-   TRANSFORM_EXPR, UNCHECKED_CONVERT_EXPR, and NULL_EXPR.  */
+   TRANSFORM_EXPR, UNCHECKED_CONVERT_EXPR, USE_EXPR and NULL_EXPR.  */
 
 static rtx
 gnat_expand_expr (exp, target, tmode, modifier)
@@ -334,6 +423,8 @@ gnat_expand_expr (exp, target, tmode, modifier)
      enum expand_modifier modifier;
 {
   tree type = TREE_TYPE (exp);
+  tree inner_type;
+  enum machine_mode mode = TYPE_MODE (type);
   tree new;
   rtx result;
 
@@ -363,23 +454,22 @@ gnat_expand_expr (exp, target, tmode, modifier)
       break;
 
     case UNCHECKED_CONVERT_EXPR:
+      inner_type = TREE_TYPE (TREE_OPERAND (exp, 0));
+
       /* If we're converting between an aggregate and non-aggregate type
 	 and we have a MEM TARGET, we can't use it, since MEM_IN_STRUCT_P
 	 would be set incorrect.  */
       if (target != 0 && GET_CODE (target) == MEM
-	  && (MEM_IN_STRUCT_P (target)
-	      != AGGREGATE_TYPE_P (TREE_TYPE (TREE_OPERAND (exp, 0)))))
+	  && (MEM_IN_STRUCT_P (target) != AGGREGATE_TYPE_P (inner_type)))
 	target = 0;
 
       /* If the input and output are both the same mode (usually BLKmode),
 	 just return the expanded input since we want just the bits.  But
 	 we can't do this if the output is more strictly aligned than
 	 the input.  */
-      if (TYPE_MODE (type) == TYPE_MODE (TREE_TYPE (TREE_OPERAND (exp, 0)))
-	  && ((TYPE_ALIGN (type)
-	       <= TYPE_ALIGN (TREE_TYPE (TREE_OPERAND (exp, 0))))
-	      || (TYPE_ALIGN (TREE_TYPE (TREE_OPERAND (exp, 0)))
-		  >= BIGGEST_ALIGNMENT)))
+      if (TYPE_MODE (type) == TYPE_MODE (inner_type)
+	  && (TYPE_ALIGN (type) <= TYPE_ALIGN (inner_type)
+	      || TYPE_ALIGN (inner_type) >= BIGGEST_ALIGNMENT))
 	new = TREE_OPERAND (exp, 0);
 
       /* If either mode is BLKmode, memory will be involved, so do this
@@ -387,13 +477,10 @@ gnat_expand_expr (exp, target, tmode, modifier)
 	 is an alignment issue.  But we must do it for types that are known
 	 to be aligned properly.  */
       else if ((TYPE_MODE (type) == BLKmode
-		|| TYPE_MODE (TREE_TYPE (TREE_OPERAND (exp, 0))) == BLKmode)
-	       && ((TYPE_ALIGN (type)
-		    <= TYPE_ALIGN (TREE_TYPE (TREE_OPERAND (exp, 0))))
-		   || (TYPE_ALIGN (TREE_TYPE (TREE_OPERAND (exp, 0)))
-		       >= BIGGEST_ALIGNMENT)
-		   || TYPE_ALIGN_OK_P (type)
-		   || TYPE_ALIGN_OK_P (TREE_TYPE (TREE_OPERAND (exp, 0)))))
+		|| TYPE_MODE (inner_type) == BLKmode)
+	       && (TYPE_ALIGN (type) <= TYPE_ALIGN (inner_type)
+		   || TYPE_ALIGN (inner_type) >= BIGGEST_ALIGNMENT
+		   || TYPE_ALIGN_OK_P (type) || TYPE_ALIGN_OK_P (inner_type)))
 	{
 	  new = build_unary_op (INDIRECT_REF, NULL_TREE,
 				convert
@@ -408,8 +495,7 @@ gnat_expand_expr (exp, target, tmode, modifier)
 	     the underlying type.  */
 	  MEM_VOLATILE_P (result) = TREE_THIS_VOLATILE (TREE_OPERAND (exp, 0));
 	  RTX_UNCHANGING_P (result) = TREE_READONLY (TREE_OPERAND (exp, 0));
-	  MEM_IN_STRUCT_P (result)
-	    = AGGREGATE_TYPE_P (TREE_TYPE (TREE_OPERAND (exp, 0)));
+	  MEM_IN_STRUCT_P (result) = AGGREGATE_TYPE_P (inner_type);
 	  return result;
 	}
 
@@ -417,17 +503,16 @@ gnat_expand_expr (exp, target, tmode, modifier)
 	 extract the other value.  */
       else
 	{
-	  tree in_type, union_type, in_field, out_field;
+	  tree union_type, in_field, out_field;
 
 	  /* If this is inside the LHS of an assignment, this would generate
 	     bad code, so abort.  */
 	  if (TREE_ADDRESSABLE (exp))
 	    gigi_abort (202);
 
-	  in_type = TREE_TYPE (TREE_OPERAND (exp, 0));
 	  union_type = make_node (UNION_TYPE);
 	  in_field = create_field_decl (get_identifier ("in"),
-					in_type, union_type, 0, 0, 0);
+					inner_type, union_type, 0, 0, 0);
 	  out_field = create_field_decl (get_identifier ("out"),
 					 type, union_type, 0, 0, 0);
 
@@ -439,8 +524,8 @@ gnat_expand_expr (exp, target, tmode, modifier)
 	     If the output size is a variable, use the input size.  */
 	  TYPE_SIZE (union_type) = TYPE_SIZE (type);
 	  if (TREE_CODE (TYPE_SIZE (type)) != INTEGER_CST
-	      && TREE_CODE (TYPE_SIZE (in_type)) == INTEGER_CST)
-	    TYPE_SIZE (union_type) = TYPE_SIZE (in_type);
+	      && TREE_CODE (TYPE_SIZE (inner_type)) == INTEGER_CST)
+	    TYPE_SIZE (union_type) = TYPE_SIZE (inner_type);
 
 	  new = build (COMPONENT_REF, type,
 		       build1 (CONVERT_EXPR, union_type,
@@ -580,6 +665,7 @@ insert_code_for (gnat_node)
     {
       push_pending_elaborations ();
       gnat_to_code (gnat_node);
+      Check_Elaboration_Code_Allowed (gnat_node);
       insert_elaboration_list (get_gnu_tree (gnat_node));
       pop_pending_elaborations ();
     }
@@ -608,14 +694,14 @@ init_lex ()
   lang_expand_expr = gnat_expand_expr;
 
   tree_code_type
-    = (char **) realloc (tree_code_type,
-			 sizeof (char *) * LAST_GNAT_TREE_CODE);
+    = (char **) xrealloc (tree_code_type,
+			  sizeof (char *) * LAST_GNAT_TREE_CODE);
   tree_code_length
-    = (int *) realloc (tree_code_length,
-		       sizeof (int) * LAST_GNAT_TREE_CODE);
+    = (int *) xrealloc (tree_code_length,
+			sizeof (int) * LAST_GNAT_TREE_CODE);
   tree_code_name
-    = (char **) realloc (tree_code_name,
-			 sizeof (char *) * LAST_GNAT_TREE_CODE);
+    = (char **) xrealloc (tree_code_name,
+			  sizeof (char *) * LAST_GNAT_TREE_CODE);
 
   bcopy ((char *) gnat_tree_code_type,
 	 (char *) (tree_code_type + (int) LAST_AND_UNUSED_TREE_CODE),
@@ -696,3 +782,110 @@ must_pass_by_ref (gnu_type)
 	  || (TYPE_SIZE (gnu_type) != 0
 	      && TREE_CODE (TYPE_SIZE (gnu_type)) != INTEGER_CST));
 }
+
+/* Convert NAME, which is possibly an Ada name, back to standard Ada
+   notation for SGI Workshop.  Return the new name. This returns NAME
+   if we aren't compiling for SGI.  */
+
+static char *
+convert_ada_name_to_qualified_name (name)
+     char *name;
+{
+#ifndef SGI_TARGET
+  return name;
+#else
+
+  int len = strlen (name);
+  char *new_name = xstrdup (name);
+  char *buf;
+  int i, start;
+  char *qual_name_suffix = 0;
+  char *p;
+
+  if (len <= 3 || use_gnu_debug_info_extensions)
+    {
+      free (new_name);
+      return name;
+    }
+
+  /* Find the position of the first "__" after the first character of
+     NAME.  This is the same as calling strstr except that we can't assume
+     the host has that function. We start after the first character so
+     we don't eliminate leading "__": these are emitted only by C
+     programs and are not qualified names */
+  for (p = (char *) index (&name[1], '_'); p != 0;
+       p = (char *) index (p+1, '_'))
+    if (p[1] == '_')
+      {
+	qual_name_suffix = p;
+	break;
+      }
+
+  if (qual_name_suffix == 0)
+    {
+      free (new_name);
+      return name;
+    }
+
+  start = qual_name_suffix - name;
+  buf = new_name + start;
+
+  for (i = start; i < len; i++)
+    {
+      if (name[i] == '_' && name[i + 1] == '_')
+	{
+	  if (islower (name[i + 2]))
+	    {
+	      *buf++ = '.';
+	      *buf++ = name[i + 2];
+	      i += 2;
+	    }
+	  else if (name[i + 2] == '_' && islower (name[i + 3]))
+	    { 
+	      /* convert foo___c___XVN to foo.c___XVN */
+	      *buf++ = '.';
+	      *buf++ = name[i + 3];
+	      i += 3;
+	    }
+	  else if (name[i + 2] == 'T')
+	    {
+	      /* convert foo__TtypeS to foo.__TTypeS */
+	      *buf++ = '.';
+	      *buf++ = '_';
+	      *buf++ = '_';
+	      *buf++ = 'T';
+	      i += 3;
+	    }
+	  else
+	    *buf++ = name[i];
+	}
+      else
+	*buf++ = name[i];
+    }
+
+  *buf = 0;
+  return new_name;
+#endif
+}
+
+/* Emit a label UNITNAME_LABEL and specify that it is part of source
+   file FILENAME.  If this is being written for SGI's Workshop
+   debugger, and we are writing Dwarf2 debugging information, add
+   additional debug info.  */
+
+void
+emit_unit_label (unitname_label, filename)
+     char *unitname_label;
+     char *filename;
+{
+  ASM_GLOBALIZE_LABEL (asm_out_file, unitname_label);
+  ASM_OUTPUT_LABEL (asm_out_file, unitname_label); 
+
+#ifdef SGI_TARGET
+#ifdef DWARF2_DEBUGGING_INFO
+  if (write_symbols == DWARF2_DEBUG)
+    dwarf2out_add_library_unit_info
+      (filename, convert_ada_name_to_qualified_name (unitname_label));
+#endif
+#endif
+}
