@@ -6,9 +6,9 @@
 /*                                                                          */
 /*                              C Header File                               */
 /*                                                                          */
-/*                            $Revision: 1.23 $                             */
+/*                            $Revision: 1.26 $                             */
 /*                                                                          */
-/*          Copyright (C) 1992-1996 Free Software Foundation, Inc.          */
+/*          Copyright (C) 1992-1999 Free Software Foundation, Inc.          */
 /*                                                                          */
 /* GNAT is free software;  you can  redistribute it  and/or modify it under */
 /* terms of the  GNU General Public License as published  by the Free Soft- */
@@ -33,9 +33,9 @@
 /****************************************************************************/
 
 /* Definitions to access the front-end's character translation tables.  */
-extern char csets__fold_lower[], csets__fold_upper[];
 #define Fold_Lower(C) csets__fold_lower[C]
 #define Fold_Upper(C) csets__fold_upper[C]
+extern char Fold_Lower[], Fold_Upper[];
 
 /* Variables expected by the gcc back-end for reference or merely linking
    purposes:  */
@@ -97,6 +97,10 @@ extern void init_lex		PROTO((void));
 /* Sets some debug flags for the parser. It does nothing here.  */
 extern void set_yydebug		PROTO((int));
 
+/* Routine called by gcc for emitting a stack check. GNU_EXPR is the
+   expression that contains the last address on the stack to check. */
+extern tree emit_stack_check    PROTO((tree));
+
 /* Utility routines created for the tree translator's sake. Their prototypes
    can be changed as desired.  */
 
@@ -112,6 +116,9 @@ extern void update_setjmp_buf (tree);
    it should be passed by reference.  */
 extern int must_pass_by_ref	     PROTO((tree));
 
+/* Subprogram to be called before generating elaboration code.  */
+#define Check_Elaboration_Code_Allowed restrict__check_elaboration_code_allowed
+extern void Check_Elaboration_Code_Allowed (Node_Id);
 
 /* elaboration routines for the front end */
 extern   void elab_all_gnat          PROTO((void));
