@@ -6,9 +6,9 @@
 /*                                                                          */
 /*                              C Header File                               */
 /*                                                                          */
-/*                            $Revision: 1.34 $                             */
+/*                            $Revision: 1.43 $
 /*                                                                          */
-/*          Copyright (C) 1992-1997 Free Software Foundation, Inc.          */
+/*          Copyright (C) 1992-1999 Free Software Foundation, Inc.          */
 /*                                                                          */
 /* GNAT is free software;  you can  redistribute it  and/or modify it under */
 /* terms of the  GNU General Public License as published  by the Free Soft- */
@@ -34,11 +34,18 @@
 #define Set_Has_No_Elab_Code sinfo__set_has_no_elab_code
 extern void Set_Has_No_Elab_Code	PROTO((Node_Id, Boolean));
 
-/* We will be setting Esize and Component_First_Bit for fields.  */
-#define Set_Esize einfo__set_esize
-extern void Set_Esize			PROTO((Entity_Id, Uint));
+/* We will be setting Esize for types, Component_First_Bit for fields,
+   Component_Size for array types, and Present_Expr for N_Variant nodes.  */
+#define Set_Esize		einfo__set_esize
 #define Set_Component_First_Bit einfo__set_component_first_bit
+#define Set_Component_Size	einfo__set_component_size
+#define Set_Present_Expr	sinfo__set_present_expr
+
+extern void Set_Component_Size		PROTO((Entity_Id, Uint));
+extern void Set_Esize			PROTO((Entity_Id, Uint));
 extern void Set_Component_First_Bit	PROTO((Entity_Id, Uint));
+extern void Set_Present_Expr		PROTO((Node_Id, Uint));
+
 
 /* These are definitions from exp_code.ads. */
 #define Is_Asm_Volatile exp_code__is_asm_volatile
@@ -81,6 +88,9 @@ extern void Next_Asm_Output PROTO((void));
 #define Ident_String lib__ident_string
 extern Node_Id Ident_String PROTO((Unit_Number_Type));
 
+#define Cunit lib__cunit
+extern Node_Id Cunit PROTO((Unit_Number_Type));
+
 /* First_Actual and Next_Actual functions from Sem_Util */
 #define First_Actual sem_util__first_actual
 extern Node_Id First_Actual PROTO((Node_Id));
@@ -94,6 +104,31 @@ extern Boolean Is_Rewrite_Subsitution PROTO((Node_Id));
 #define Original_Node atree__original_node
 extern Node_Id Original_Node PROTO((Node_Id));
 
+/* Flags from opt.ads, debug.ads, and hostparm.ads.  */
+#define Global_Discard_Names opt__global_discard_names
+extern Boolean Global_Discard_Names;
+
+#define Debug_Flag_XX debug__debug_flag_xx
+#define Debug_Flag_NN debug__debug_flag_nn
+extern Boolean Debug_Flag_XX, Debug_Flag_NN;
+
+#define Stack_Check_Probes_Val opt__stack_check_probes_val
+extern Boolean Stack_Check_Probes_Val;
+
+#define Zero_Cost_Exceptions_Val opt__zero_cost_exceptions_val
+extern Boolean Zero_Cost_Exceptions_Val;
+
+/* Flag indicating whether file names are discarded in exception messages */
+extern int discard_file_names;
+
+/* If true, then gigi is being called on an analyzed but unexpanded
+   tree, and the only purpose of the call is to properly annotate
+   types with representation information */
+extern int type_annotate_only;
+
+/* Current file name without path */
+extern char *ref_filename;
+
 /* List of TREE_LIST nodes representing a block stack.  TREE_VALUE
    of each gives the variable used for the setjmp buffer in the current
    block, if any.  */
@@ -150,6 +185,9 @@ extern void post_error_ne_tree	PROTO((char *, Node_Id, Entity_Id, tree));
 extern void post_error_ne_tree_2 PROTO((char *, Node_Id, Entity_Id,
 					tree, int));
 
+/* Set the node for a second '&' in the error message.  */
+extern void set_second_error_entity PROTO((Entity_Id));
+
 /* Surround EXP with a SAVE_EXPR, but handle unconstrained objects specially
    since it doesn't make any sense to put them in a SAVE_EXPR.  */
 extern tree make_save_expr	PROTO((tree));
@@ -157,13 +195,19 @@ extern tree make_save_expr	PROTO((tree));
 /* Signal abort, with "Gigi abort" as the error label, and error_gnat_node
    as the relevant node that provides the location info for the error.
    The single parameter CODE is an integer code that is included in the
-   additional error message generated */
-extern void gigi_abort          PROTO((int))  __attribute__ ((noreturn));
+   additional error message generated. Note that a call to gigi_abort
+   has no effect in type_annotate_only mode. This is an attempt to avoid
+   some of the crashes that this special mode may cause */
+extern void gigi_abort          PROTO((int)) __attribute__ ((noreturn));
 
 /* Initialize the table that maps GNAT codes to GCC codes for simple
    binary and unary operations.  */
 extern void init_code_table	PROTO((void));
 
+/* Current node being treated, in case gigi_abort or Check_Elaboration_Code
+   called.  */
+extern Node_Id error_gnat_node;
+
 /* This is equivalent to stabilize_reference in GCC's tree.c, but we know
    how to handle our new nodes and we take an extra argument that says 
    whether to force evaluation of everything.  */
