@@ -6,9 +6,9 @@
 /*                                                                          */
 /*                              C Header File                               */
 /*                                                                          */
-/*                            $Revision: 1.108 $                            */
+/*                            $Revision: 1.121 $
 /*                                                                          */
-/*          Copyright (C) 1992-1997 Free Software Foundation, Inc.          */
+/*          Copyright (C) 1992-1999 Free Software Foundation, Inc.          */
 /*                                                                          */
 /* GNAT is free software;  you can  redistribute it  and/or modify it under */
 /* terms of the  GNU General Public License as published  by the Free Soft- */
@@ -70,8 +70,8 @@ extern int force_global;
 
 /* Flags added to GCC type nodes.  */
 
-/* For RECORD_TYPE, nonzero if this is a record being used
-   as a fat pointer.  */
+/* For RECORD_TYPE, UNION_TYPE, and QUAL_UNION_TYPE, nonzero if this is a
+   record being used as a fat pointer (only true for RECORD_TYPE).  */
 #define TYPE_IS_FAT_POINTER_P(NODE) TYPE_LANG_FLAG_0 (NODE)
 
 #define TYPE_FAT_POINTER_P(NODE)  \
@@ -93,7 +93,8 @@ extern int force_global;
    unconstrained array or record.  */
 #define TYPE_RETURNS_UNCONSTRAINED_P(NODE) TYPE_LANG_FLAG_1 (NODE)
 
-/* For RECORD_TYPE, nonzero if this denotes a left-justified modular type.  */
+/* For RECORD_TYPE, UNION_TYPE, and QUAL_UNION_TYPE, nonzero if this denotes
+   a left-justified modular type (will only be true for RECORD_TYPE).  */
 #define TYPE_LEFT_JUSTIFIED_MODULAR_P(NODE) TYPE_LANG_FLAG_1 (NODE)
 
 /* Nonzero in an arithmetic subtype if this is a subtype not known to the
@@ -103,8 +104,9 @@ extern int force_global;
 /* Nonzero for composite types if this is a by-reference type.  */
 #define TYPE_BY_REFERENCE_P(NODE) TYPE_LANG_FLAG_2 (NODE)
 
-/* For RECORD_TYPE, nonzero if this is the type for an object whose type
-   includes its template in addition to its value.  */
+/* For RECORD_TYPE, UNION_TYPE, and QUAL_UNION_TYPE, nonzero if this is the
+   type for an object whose type includes its template in addition to
+   its value (only true for RECORD_TYPE).  */
 #define TYPE_CONTAINS_TEMPLATE_P(NODE) TYPE_LANG_FLAG_3 (NODE)
 
 /* For INTEGER_TYPE, nonzero if this really represents a VAX
@@ -113,7 +115,7 @@ extern int force_global;
 
 /* True if NODE is a thin pointer.  */
 #define TYPE_THIN_POINTER_P(NODE)			\
-  (TREE_CODE (NODE) == POINTER_TYPE			\
+  (POINTER_TYPE_P (NODE)				\
    && TREE_CODE (TREE_TYPE (NODE)) == RECORD_TYPE	\
    && TYPE_CONTAINS_TEMPLATE_P (TREE_TYPE (NODE)))
 
@@ -202,7 +204,7 @@ extern int force_global;
    the template and object.  */
 #define TYPE_OBJECT_RECORD_TYPE(NODE) TYPE_MIN_VALUE (NODE)
 
-/* Nonzero in a FUCNTION_DECL that represents a stubbed function
+/* Nonzero in a FUNCTION_DECL that represents a stubbed function
    discriminant.  */
 #define DECL_STUBBED_P(NODE) DECL_LANG_FLAG_0 (NODE)
 
@@ -217,6 +219,9 @@ extern int force_global;
 /* Nonzero in a FIELD_DECL that is a dummy built for some internal reason.  */
 #define DECL_INTERNAL_P(NODE) DECL_LANG_FLAG_3 (NODE)
 
+/* Nonzero in a FUNCTION_DECL that corresponds to an elaboration procedure.  */
+#define DECL_ELABORATION_PROC_P(NODE) DECL_LANG_FLAG_3 (NODE)
+
 /* Nonzero if this is a decl for a pointer that points to something which
    is readonly.  Used mostly for fat pointers.  */
 #define DECL_POINTS_TO_READONLY_P(NODE) DECL_LANG_FLAG_4 (NODE)
@@ -232,6 +237,14 @@ extern int force_global;
    address taken.  */
 #define DECL_CONST_CORRESPONDING_VAR(NODE) (tree) DECL_LANG_SPECIFIC (NODE)
 
+/* In a FIELD_DECL, points to the FIELD_DECL that was the ultimate
+   source of the decl.  */
+#define DECL_ORIGINAL_FIELD(NODE) (tree) DECL_LANG_SPECIFIC (NODE)
+
+/* In a FIELD_DECL corresponding to a discriminant, contains the
+   discriminant number.  */
+#define DECL_DISCRIMINANT_NUMBER(NODE) DECL_INITIAL (NODE)
+
 /* Nonzero in a TRANSFORM_EXPR if we are to take its address.  */
 #define TREE_TRANSFORM_ADDR(NODE) TREE_LANG_FLAG_0 (NODE)
 
@@ -292,21 +305,16 @@ extern tree malloc_decl;
 /* Likewise for freeing memory.  */
 extern tree free_decl;
 
-/* Likewise for setting the current exception occurrence */
-extern tree set_except_occ_decl;
-
 /* Types and decls used by our temporary exception mechanism.  See
-   init_decl_processing for details.  */
+   init_gigi_decls for details.  */
 extern tree jmpbuf_type;
 extern tree jmpbuf_ptr_type;
 extern tree get_jmpbuf_decl;
 extern tree set_jmpbuf_decl;
 extern tree get_excptr_decl;
-extern tree raise_decl;
-extern tree raise_with_msg_decl;
-extern tree raise_nodefer_decl;
 extern tree setjmp_decl;
 extern tree longjmp_decl;
+extern tree raise_nodefer_decl;
 extern tree raise_constraint_error_decl;
 extern tree raise_program_error_decl;
 extern tree raise_storage_error_decl;
@@ -360,6 +368,7 @@ extern tree pushdecl			PROTO((tree));
 /* Create the predefined scalar types such as `integer_type_node' needed 
    in the gcc back-end and initialize the global binding level.  */
 extern void init_decl_processing	PROTO((void));
+extern void init_gigi_decls		PROTO((tree, tree));
 
 /* Return an integer type with the number of bits of precision given by  
    PRECISION.  UNSIGNEDP is nonzero if the type is unsigned; otherwise
@@ -444,9 +453,13 @@ extern tree copy_type			PROTO((tree));
    TYPE_INDEX_TYPE is INDEX.  */
 extern tree create_index_type		PROTO((tree, tree, tree));
 
-/* Returns a TYPE_DECL node. TYPE_NAME gives the name of the type and TYPE
-   is a ..._TYPE node giving its data type.  */
-extern tree create_type_decl		PROTO((tree, tree, struct attrib *));
+/* Return a TYPE_DECL node. TYPE_NAME gives the name of the type (a character
+   string) and TYPE is a ..._TYPE node giving its data type. 
+   ARTIFICIAL_P is nonzero if this is a declaration that was generated
+   by the compiler.  DEBUG_INFO_P is nonzero if we need to write debugging
+   information about this type.  */
+extern tree create_type_decl		PROTO((tree, tree, struct attrib *,
+					       int, int));
 
 /* Returns a GCC VAR_DECL node. VAR_NAME gives the name of the variable.
    ASM_NAME is its assembler name (if provided).  TYPE is
@@ -556,6 +569,10 @@ extern void update_pointer_to	PROTO((tree, tree));
    minimum (if ! MAX_P) possible value of the discriminant.  */
 extern tree max_size		PROTO((tree, int));
 
+/* Remove all conversions that are done in EXP.  This includes converting
+   from a padded type or converting to a left-justified modular type.  */
+extern tree remove_conversions	PROTO((tree));
+
 /* If EXP's type is an UNCONSTRAINED_ARRAY_TYPE, return an expression that
    refers to the underlying array.  If its type has TYPE_CONTAINS_TEMPLATE_P,
    likewise return an expression pointing to the underlying array.  */
