@@ -6,9 +6,9 @@
 /*                                                                          */
 /*                          C Implementation File                           */
 /*                                                                          */
-/*                            $Revision: 1.148 $                            */
+/*                            $Revision: 1.172 $                            */
 /*                                                                          */
-/*          Copyright (C) 1992-1997, Free Software Foundation, Inc.         */
+/*          Copyright (C) 1992-1999, Free Software Foundation, Inc.         */
 /*                                                                          */
 /* GNAT is free software;  you can  redistribute it  and/or modify it under */
 /* terms of the  GNU General Public License as published  by the Free Soft- */
@@ -47,9 +47,11 @@
 #include "a-misc.h"
 
 static tree find_common_type		PROTO((tree, tree));
+static int contains_save_expr_p		PROTO((tree));
 static tree compare_arrays		PROTO((tree, tree, tree));
 static tree nonbinary_modular_operation	PROTO((enum tree_code, tree,
 					       tree, tree));
+static tree build_simple_component_ref	PROTO((tree, tree, tree));
 
 /* Prepare expr to be an argument of a TRUTH_NOT_EXPR or other logical
    operation.
@@ -90,6 +92,11 @@ truthvalue_conversion (expr)
       return fold (build (COND_EXPR, type, TREE_OPERAND (expr, 0),
 			  truthvalue_conversion (TREE_OPERAND (expr, 1)),
 			  truthvalue_conversion (TREE_OPERAND (expr, 2))));
+
+    case WITH_RECORD_EXPR:
+      return build (WITH_RECORD_EXPR, type,
+		    truthvalue_conversion (TREE_OPERAND (expr, 0)),
+		    TREE_OPERAND (expr, 1));
     }
 
   return build_binary_op (NE_EXPR, type, expr,
@@ -128,6 +135,61 @@ get_ada_base_type (type)
   return type;
 }
 
+/* EXP is a GCC tree representing an address.  See if we can find how
+   strictly the object at that address is aligned.   Return that alignment
+   in bits.  If we don't know anything about the alignment, return 0.
+   We do not go merely by type information here since the check on
+   N_Validate_Unchecked_Alignment does that.  */
+
+int
+known_alignment (exp)
+     tree exp;
+{
+  int lhs, rhs;
+
+  switch (TREE_CODE (exp))
+    {
+    case CONVERT_EXPR:
+    case NOP_EXPR:
+    case NON_LVALUE_EXPR:
+      /* Conversions between pointers and integers don't change the alignment
+	 of the underlying object.  */
+      return known_alignment (TREE_OPERAND (exp, 0));
+
+    case PLUS_EXPR:
+    case MINUS_EXPR:
+      /* If two address are added, the alignment of the result is the
+	 minimum of the two aligments.  */
+      lhs = known_alignment (TREE_OPERAND (exp, 0));
+      rhs = known_alignment (TREE_OPERAND (exp, 1));
+      return MIN (lhs, rhs);
+
+    case INTEGER_CST:
+      /* The first part of this represents the lowest bit in the constant,
+	 but is it in bytes, not bits.  */
+      return MIN (BITS_PER_UNIT
+		  * (TREE_INT_CST_LOW (exp) & - TREE_INT_CST_LOW (exp)),
+		  BIGGEST_ALIGNMENT);
+
+    case MULT_EXPR:
+      /* If we know the alignment of just one side, use it.  Otherwise,
+	 use the product of the alignments.  */
+      lhs = known_alignment (TREE_OPERAND (exp, 0));
+      rhs = known_alignment (TREE_OPERAND (exp, 1));
+      if (lhs == 0 || rhs == 0)
+	return MIN (BIGGEST_ALIGNMENT, MAX (lhs, rhs));
+
+      return MIN (BIGGEST_ALIGNMENT, lhs * rhs);
+
+    case ADDR_EXPR:
+      if (TREE_CODE_CLASS (TREE_CODE (TREE_OPERAND (exp, 0))) == 'd')
+	return DECL_ALIGN (TREE_OPERAND (exp, 0));
+      break;
+    }
+
+  return 0;
+}
+
 /* We have a comparison or assignment operation on two types, T1 and T2,
    which are both either array types or both record types.
    Return the type that both operands should be converted to, if any.
@@ -157,6 +219,40 @@ find_common_type (t1, t2)
   return 0;
 }
 
+/* See if EXP contains a SAVE_EXPR in a position where we would
+   normally put it.
+
+   ??? This is a real kludge, but is probably the best approach short
+   of some very general solution.  */
+
+static int
+contains_save_expr_p (exp)
+     tree exp;
+{
+  switch (TREE_CODE (exp))
+    {
+    case SAVE_EXPR:
+      return 1;
+
+    case ADDR_EXPR:  case INDIRECT_REF:
+    case COMPONENT_REF:
+    case NOP_EXPR:  case CONVERT_EXPR: case UNCHECKED_CONVERT_EXPR:
+      return contains_save_expr_p (TREE_OPERAND (exp, 0));
+
+    case CONSTRUCTOR:
+      return (CONSTRUCTOR_ELTS (exp) != 0
+	      && contains_save_expr_p (CONSTRUCTOR_ELTS (exp)));
+
+    case TREE_LIST:
+      return (contains_save_expr_p (TREE_VALUE (exp))
+	      || (TREE_CHAIN (exp) != 0
+		  && contains_save_expr_p (TREE_CHAIN (exp))));
+    }
+
+  return 0;
+}
+
+
 /* Return an expression tree representing an equality comparison of
    A1 and A2, two objects of ARRAY_TYPE.  The returned expression should
    be of type RESULT_TYPE
@@ -192,7 +288,7 @@ compare_arrays (result_type, a1, a2)
       tree length2 = fold (build (MINUS_EXPR, bt, ub2, lb2));
       tree nbt;
       tree tem;
-      tree comparison;
+      tree comparison, this_a1_is_null, this_a2_is_null;
 
       /* If the length of the first array is a constant, swap our operands
 	 unless the length of the second array is the constant zero.  
@@ -227,6 +323,9 @@ compare_arrays (result_type, a1, a2)
 	    length1 = build (WITH_RECORD_EXPR, bt, length1, a1);
 
 	  length_zero_p = 1;
+
+	  this_a1_is_null = comparison;
+	  this_a2_is_null = convert (result_type, integer_one_node);
 	}
 
       /* If the length is some other constant value, we know that the
@@ -252,6 +351,9 @@ compare_arrays (result_type, a1, a2)
 	    comparison = build (WITH_RECORD_EXPR, result_type, comparison, a1);
 	  if (contains_placeholder_p (length1))
 	    length1 = build (WITH_RECORD_EXPR, bt, length1, a1);
+
+	  this_a1_is_null = build_binary_op (LT_EXPR, result_type, ub1, lb1);
+	  this_a2_is_null = convert (result_type, integer_zero_node);
 	}
 
       /* Otherwise compare the computed lengths.  */
@@ -264,20 +366,22 @@ compare_arrays (result_type, a1, a2)
 
 	  comparison
 	    = build_binary_op (EQ_EXPR, result_type, length1, length2);
+
+	  this_a1_is_null
+	    = build_binary_op (LT_EXPR, result_type, length1,
+			       convert (bt, integer_zero_node));
+	  this_a2_is_null
+	    = build_binary_op (LT_EXPR, result_type, length2,
+			       convert (bt, integer_zero_node));
 	}
 
       result = build_binary_op (TRUTH_ANDIF_EXPR, result_type,
 				result, comparison);
-      a1_is_null
-	= build_binary_op (TRUTH_ORIF_EXPR, result_type,
-			   build_binary_op (LT_EXPR, result_type, length1,
-					    convert (bt, integer_zero_node)),
-			   a1_is_null);
-      a2_is_null
-	= build_binary_op (TRUTH_ORIF_EXPR, result_type,
-			   build_binary_op (LT_EXPR, result_type, length2,
-					    convert (bt, integer_zero_node)),
-			   a2_is_null);
+
+      a1_is_null = build_binary_op (TRUTH_ORIF_EXPR, result_type,
+				    this_a1_is_null, a1_is_null);
+      a2_is_null = build_binary_op (TRUTH_ORIF_EXPR, result_type,
+				    this_a2_is_null, a2_is_null);
 
       t1 = TREE_TYPE (t1);
       t2 = TREE_TYPE (t2);
@@ -293,11 +397,12 @@ compare_arrays (result_type, a1, a2)
 	a1 = convert (type, a1), a2 = convert (type, a2);
 
 
-      result
-	= build_binary_op (TRUTH_ANDIF_EXPR, result_type, result,
-			   build (EQ_EXPR, result_type, a1, a2));
+      result = build_binary_op (TRUTH_ANDIF_EXPR, result_type, result,
+				build (EQ_EXPR, result_type, a1, a2));
+
     }
 
+  /* The result is also true if both sizes are zero.  */
   result = build_binary_op (TRUTH_ORIF_EXPR, result_type,
 			    build_binary_op (TRUTH_ANDIF_EXPR, result_type,
 					     a1_is_null, a2_is_null),
@@ -306,20 +411,11 @@ compare_arrays (result_type, a1, a2)
   /* If either operand contains SAVE_EXPRs, they have to be evaluated before
      starting the comparison above since the place it would be otherwise
      evaluated would be wrong.  */
-  if (TREE_CODE (a1) == SAVE_EXPR
-      || (TREE_CODE (a1) == INDIRECT_REF
-	  && TREE_CODE (TREE_OPERAND (a1, 0)) == SAVE_EXPR)
-      || (TREE_CODE (a1) == INDIRECT_REF
-	  && TREE_CODE (TREE_OPERAND (a1, 0)) == COMPONENT_REF
-	  && TREE_CODE (TREE_OPERAND (TREE_OPERAND (a1, 0), 0)) == SAVE_EXPR))
+
+  if (contains_save_expr_p (a1))
     result = build (COMPOUND_EXPR, result_type, a1, result);
 
-  if (TREE_CODE (a2) == SAVE_EXPR
-      || (TREE_CODE (a2) == INDIRECT_REF
-	  && TREE_CODE (TREE_OPERAND (a2, 0)) == SAVE_EXPR)
-      || (TREE_CODE (a2) == INDIRECT_REF
-	  && TREE_CODE (TREE_OPERAND (a2, 0)) == COMPONENT_REF
-	  && TREE_CODE (TREE_OPERAND (TREE_OPERAND (a2, 0), 0)) == SAVE_EXPR))
+  if (contains_save_expr_p (a2))
     result = build (COMPOUND_EXPR, result_type, a2, result);
 
   return result;
@@ -450,6 +546,37 @@ build_binary_op (op_code, result_type, left_operand, right_operand)
   tree result;
   int has_side_effects = 0;
 
+  /* If one (but not both, unless they have the same object) operands are a
+     WITH_RECORD_EXPR, do the operation and then surround it with the
+     WITH_RECORD_EXPR.  Don't do this for assignment or for an ARRAY_REF
+     because we need to keep track of the WITH_RECORD_EXPRs on both
+     operands very carefully.  */
+  if (op_code != MODIFY_EXPR && op_code != ARRAY_REF
+      && TREE_CODE (left_operand) == WITH_RECORD_EXPR
+      && (TREE_CODE (right_operand) != WITH_RECORD_EXPR
+	  || operand_equal_p (TREE_OPERAND (left_operand, 1),
+			      TREE_OPERAND (right_operand, 1), 0)))
+    {
+      tree right = right_operand;
+
+      if (TREE_CODE (right) == WITH_RECORD_EXPR)
+	right = TREE_OPERAND (right, 0);
+
+      result = build_binary_op (op_code, result_type,
+				TREE_OPERAND (left_operand, 0), right);
+      return build (WITH_RECORD_EXPR, TREE_TYPE (result), result,
+		    TREE_OPERAND (left_operand, 1));
+    }
+  else if (op_code != MODIFY_EXPR && op_code != ARRAY_REF
+	   && TREE_CODE (left_operand) != WITH_RECORD_EXPR
+	   && TREE_CODE (right_operand) == WITH_RECORD_EXPR)
+    {
+      result = build_binary_op (op_code, result_type, left_operand,
+				TREE_OPERAND (right_operand, 0));
+      return build (WITH_RECORD_EXPR, TREE_TYPE (result), result,
+		    TREE_OPERAND (right_operand, 1));
+    }
+
   if (operation_type != 0
       && TREE_CODE (operation_type) == RECORD_TYPE
       && TYPE_LEFT_JUSTIFIED_MODULAR_P (operation_type))
@@ -551,6 +678,7 @@ build_binary_op (op_code, result_type, left_operand, right_operand)
 	   || TREE_CODE (result) == ARRAY_REF
 	   || TREE_CODE (result) == REALPART_EXPR
 	   || TREE_CODE (result) == IMAGPART_EXPR
+	   || TREE_CODE (result) == WITH_RECORD_EXPR
 	   || TREE_CODE (result) == UNCHECKED_CONVERT_EXPR
 	   || (TREE_CODE (result) == NOP_EXPR
 	       && (TREE_CODE (TREE_TYPE (result))
@@ -614,7 +742,7 @@ build_binary_op (op_code, result_type, left_operand, right_operand)
     case LE_EXPR:
     case GT_EXPR:
     case LT_EXPR:
-      if (TREE_CODE (left_type) == POINTER_TYPE)
+      if (POINTER_TYPE_P (left_type))
 	gigi_abort (501);
 
       /* ... fall through ... */
@@ -622,13 +750,18 @@ build_binary_op (op_code, result_type, left_operand, right_operand)
     case EQ_EXPR:
     case NE_EXPR:
       /* If either operand is a NULL_EXPR, just return a new one.  */
-      if (TREE_CODE (left_operand) == NULL_EXPR
-	  || TREE_CODE (right_operand) == NULL_EXPR)
+      if (TREE_CODE (left_operand) == NULL_EXPR)
 	return build (op_code, result_type,
 		      build1 (NULL_EXPR, integer_type_node,
 			      TREE_OPERAND (left_operand, 0)),
 		      integer_zero_node);
 
+      else if (TREE_CODE (right_operand) == NULL_EXPR)
+	return build (op_code, result_type,
+		      build1 (NULL_EXPR, integer_type_node,
+			      TREE_OPERAND (right_operand, 0)),
+		      integer_zero_node);
+
       /* If either object is a left-justified modular types, get the
 	 fields from within.  */
       if (TREE_CODE (left_type) == RECORD_TYPE
@@ -817,6 +950,11 @@ build_binary_op (op_code, result_type, left_operand, right_operand)
 					    left_operand, right_operand);
       modulus = 0;
     }
+  /* If either operand is a NULL_EXPR, just return a new one.  */
+  else if (TREE_CODE (left_operand) == NULL_EXPR)
+    return build1 (NULL_EXPR, operation_type, TREE_OPERAND (left_operand, 0));
+  else if (TREE_CODE (right_operand) == NULL_EXPR)
+    return build1 (NULL_EXPR, operation_type, TREE_OPERAND (right_operand, 0));
   else
     result = fold (build (op_code, operation_type,
 			  left_operand, right_operand));
@@ -852,6 +990,17 @@ build_unary_op (op_code, result_type, operand)
   tree result;
   int side_effects = 0;
 
+  /* If we have a WITH_RECORD_EXPR as our operand, do the operation first,
+     then surround it with the WITH_RECORD_EXPR.  This allows GCC to do better
+     expression folding.  */
+  if (TREE_CODE (operand) == WITH_RECORD_EXPR)
+    {
+      result = build_unary_op (op_code, result_type,
+			       TREE_OPERAND (operand, 0));
+      return build (WITH_RECORD_EXPR, TREE_TYPE (result), result,
+		    TREE_OPERAND (operand, 1));
+    }
+
   if (operation_type != 0
       && TREE_CODE (operation_type) == RECORD_TYPE
       && TYPE_LEFT_JUSTIFIED_MODULAR_P (operation_type))
@@ -881,12 +1030,18 @@ build_unary_op (op_code, result_type, operand)
       result = invert_truthvalue (truthvalue_conversion (operand));
       break;
 
+    case ATTR_ADDR_EXPR:
     case ADDR_EXPR:
       switch (TREE_CODE (operand))
 	{
 	case INDIRECT_REF:
 	case UNCONSTRAINED_ARRAY_REF:
 	  result = TREE_OPERAND (operand, 0);
+
+	  /* Make sure the type here is a pointer, not a reference.
+	     GCC wants pointer types for function addresses.  */
+	  if (result_type == 0)
+	    result_type = build_pointer_type (type);
 	  break;
 
 	case NULL_EXPR:
@@ -906,51 +1061,96 @@ build_unary_op (op_code, result_type, operand)
 	  break;
 
 	case ARRAY_REF:
+	case COMPONENT_REF:
+	case BIT_FIELD_REF:
 	  {
-	    /* Compute address of array element by adding address of array
-	       to index times size of element.  */
-	    tree array = TREE_OPERAND (operand, 0);
-	    tree array_adr = build1 (ADDR_EXPR,
-				     build_pointer_type (type), array);
-	    tree low_bound = TYPE_MIN_VALUE (TYPE_DOMAIN (TREE_TYPE (array)));
-	    tree index = TREE_OPERAND (operand, 1);
-	    tree size = size_in_bytes (type);
+	    tree op_type = TREE_TYPE (TREE_OPERAND (operand, 0));
+	    int bitsize, bitpos;
+	    tree offset, inner;
+	    enum machine_mode mode;
+	    int unsignedp, volatilep, alignment;
 
-	    mark_addressable (array);
+	    inner = get_inner_reference (operand, &bitsize, &bitpos, &offset,
+					 &mode, &unsignedp, &volatilep,
+					 &alignment);
 
-	    if (TREE_CODE (low_bound) != INTEGER_CST
-		&& contains_placeholder_p (low_bound))
-	      low_bound = build (WITH_RECORD_EXPR, sizetype,
-				 low_bound, operand);
+	    /* If this is a bitfield and not part of the 'Address
+	       attribute, let this be done the normal way, which will make a 
+	       temporary containing the value and take the address of that.
+	       Likewise if we didn't find anything different for the inner
+	       operand or if OPERAND extracts direction from a padding type
+	       whose field has a self-referential size (to avoid infinite
+	       recursion below).  */
+	    if ((mode == VOIDmode && op_code != ATTR_ADDR_EXPR)
+		|| inner == operand
+		|| (TREE_CODE (op_type) == RECORD_TYPE
+		    && TYPE_IS_PADDING_P (op_type)
+		    && (contains_placeholder_p
+			(TYPE_SIZE (TREE_TYPE (TYPE_FIELDS (op_type)))))))
+	      goto common;
 
-	    /* Convert the integer argument to a type the same size as a
-	       pointer so the arithmetic won't overflow spuriously.  */
-	    if (TYPE_PRECISION (TREE_TYPE (index)) != POINTER_SIZE)
-	      index = convert (type_for_size (POINTER_SIZE, 0), index);
+	    /* If INNER is a padding type whose field has a self-referential
+	       size, convert to that inner type.  We know the offset is zero
+	       and we need to have that type visible.  */
+	    if (TREE_CODE (TREE_TYPE (inner)) == RECORD_TYPE
+		&& TYPE_IS_PADDING_P (TREE_TYPE (inner))
+		&& (contains_placeholder_p
+		    (TYPE_SIZE (TREE_TYPE (TYPE_FIELDS (TREE_TYPE (inner)))))))
+	      inner = convert (TREE_TYPE (TYPE_FIELDS (TREE_TYPE (inner))),
+			       inner);
 
-	    index = fold (build (MINUS_EXPR, TREE_TYPE (index), index,
-				 convert (TREE_TYPE (index), low_bound)));
+	    /* Compute the offset as a byte offset from INNER.  */
+	    if (offset == 0)
+	      offset = size_zero_node;
 
-	    index = fold (build (MULT_EXPR, TREE_TYPE (index),
-				 index, convert (TREE_TYPE (index), size)));
+	    offset = size_binop (PLUS_EXPR, offset,
+				 size_int (bitpos / BITS_PER_UNIT));
 
-	    result = fold (build (PLUS_EXPR, TYPE_POINTER_TO (type), array_adr,
-				  convert (TYPE_POINTER_TO (type), index)));
+	    /* Take the address of INNER, convert the offset to void *, and
+	       add then.  It will later be converted to the desired result
+	       type, if any.  */
+	    inner = build_unary_op (ADDR_EXPR, NULL_TREE, inner);
+	    inner = convert (ptr_void_type_node, inner);
+	    offset = convert (ptr_void_type_node, offset);
+	    result = build_binary_op (PLUS_EXPR, ptr_void_type_node,
+				      inner, offset);
+	    result = convert (build_pointer_type (TREE_TYPE (operand)),
+			      result);
 	  }
 	  break;
 
+	case NOP_EXPR:
+	  if (AGGREGATE_TYPE_P (type)
+	      && AGGREGATE_TYPE_P (TREE_TYPE (TREE_OPERAND (operand, 0))))
+	    return build_unary_op (ADDR_EXPR, result_type,
+				   TREE_OPERAND (operand, 0));
+
+	  /* If this NOP_EXPR doesn't change the mode, get the result type
+	     from this type and go down.  We need to do this in case
+	     this is a conversion of a CONST_DECL.  */
+	  if (TYPE_MODE (type) != BLKmode
+	      && (TYPE_MODE (type)
+		  == TYPE_MODE (TREE_TYPE (TREE_OPERAND (operand, 0)))))
+	    return build_unary_op (ADDR_EXPR,
+				   (result_type == 0
+				    ? build_pointer_type (type)
+				    : result_type),
+				   TREE_OPERAND (operand, 0));
+	  goto common;
+
 	case CONST_DECL:
 	  operand = DECL_CONST_CORRESPONDING_VAR (operand);
 
 	  /* ... fall through ... */
 
 	default:
+	common:
 
 	  if (type != error_mark_node)
 	    operation_type = build_pointer_type (type);
 
 	  mark_addressable (operand);
-	  result = fold (build1 (op_code, operation_type, operand));
+	  result = fold (build1 (ADDR_EXPR, operation_type, operand));
 	}
 
       TREE_CONSTANT (result) = staticp (operand) || TREE_CONSTANT (operand);
@@ -973,6 +1173,10 @@ build_unary_op (op_code, result_type, operand)
       if (TYPE_FAT_POINTER_P (type))
 	result = build1 (UNCONSTRAINED_ARRAY_REF,
 			 TYPE_UNCONSTRAINED_ARRAY (type), operand);
+
+      else if (TREE_CODE (operand) == ADDR_EXPR)
+	result = TREE_OPERAND (operand, 0);
+
       else
 	{
 	  result = fold (build1 (op_code, TREE_TYPE (type), operand));
@@ -1122,6 +1326,7 @@ build_cond_expr (result_type, condition_operand, true_operand, false_operand)
 
 /* Build a CALL_EXPR to call FUNDECL with one argument, ARG.  Return
    the CALL_EXPR.  */
+
 tree
 build_call_1_expr (fundecl, arg)
      tree fundecl;
@@ -1137,6 +1342,26 @@ build_call_1_expr (fundecl, arg)
   return call;
 }
 
+/* Build a CALL_EXPR to call FUNDECL with two arguments, ARG1 & ARG2.  Return
+   the CALL_EXPR.  */
+
+tree
+build_call_2_expr (fundecl, arg1, arg2)
+     tree fundecl;
+     tree arg1, arg2;
+{
+  tree call = build (CALL_EXPR, TREE_TYPE (TREE_TYPE (fundecl)),
+		     build_unary_op (ADDR_EXPR, NULL_TREE, fundecl),
+		     chainon (chainon (NULL_TREE,
+				       build_tree_list (NULL_TREE, arg1)),
+			      build_tree_list (NULL_TREE, arg2)),
+		     NULL_TREE);
+
+  TREE_SIDE_EFFECTS (call) = 1;
+
+  return call;
+}
+
 /* Likewise to call FUNDECL with no arguments.  */
 
 tree
@@ -1152,7 +1377,28 @@ build_call_0_expr (fundecl)
   return call;
 }
 
+/* Call a function FCN that raises an exception and pass the line
+   number and file name, if requested.  */
 
+tree
+build_call_raise (fndecl)
+     tree fndecl;
+{
+  char *str = discard_file_names ? "" : ref_filename;
+  int len = strlen (str) + 1;
+  tree filename = build_string (len, str);
+
+  TREE_TYPE (filename)
+    = build_array_type (char_type_node,
+			build_index_type (build_int_2 (len, 0)));
+
+  return
+    build_call_2_expr (fndecl,
+		       build1 (ADDR_EXPR, build_pointer_type (char_type_node),
+			       filename),
+		       build_int_2 (lineno, 0));
+}
+
 /* Return a CONSTRUCTOR of TYPE whose list is LIST.  */
 
 tree
@@ -1161,7 +1407,7 @@ build_constructor (type, list)
      tree list;
 {
   tree elmt;
-  int allconstant = 1;
+  int allconstant = (TREE_CODE (TYPE_SIZE (type)) == INTEGER_CST);
   int side_effects = 0;
   tree result;
 
@@ -1208,13 +1454,13 @@ build_constructor (type, list)
 }
 
 /* Return a COMPONENT_REF to access a field that is given by COMPONENT,
-   an IDENTIFIER_NODE giving the name of the field, FIELD, a FIELD_DECL,
-   for the field, or both.
+   an IDENTIFIER_NODE giving the name of the field, or FIELD, a FIELD_DECL,
+   for the field.
 
    We also handle the fact that we might have been passed a pointer to the
    actual record and know how to look for fields in variant parts.  */
 
-tree
+static tree
 build_simple_component_ref (record_variable, component, field)
      tree record_variable;
      tree component;
@@ -1223,34 +1469,43 @@ build_simple_component_ref (record_variable, component, field)
   tree record_type = TYPE_MAIN_VARIANT (TREE_TYPE (record_variable));
   tree ref;
 
-  /* Handle added pointer for pass-by-reference values.  */
-  if (TREE_CODE (record_type) == POINTER_TYPE)
-    {
-      record_variable
-	= build_unary_op (INDIRECT_REF, NULL_TREE, record_variable);
-      record_type = TREE_TYPE (record_variable);
-    }
-
   if ((TREE_CODE (record_type) != RECORD_TYPE
        && TREE_CODE (record_type) != UNION_TYPE
        && TREE_CODE (record_type) != QUAL_UNION_TYPE)
       || TYPE_SIZE (record_type) == 0)
     gigi_abort (510);
 
-  if (field == 0 || DECL_CONTEXT (field) != record_type)
+  /* Either COMPONENT or FIELD must be specified, but not both.  */
+  if ((component != 0) == (field != 0))
+    gigi_abort (511);
+
+  /* If no field was specified, look for a field with the specified name
+     in the current record only.  */
+  if (field == 0)
+    for (field = TYPE_FIELDS (record_type); field;
+	 field = TREE_CHAIN (field))
+      if (DECL_NAME (field) == component)
+	break;
+
+  if (field == 0)
+    return 0;
+
+  /* If this field is not in the specified record, see if we can find
+     something in the record whose original field is the same as this one. */
+  if (DECL_CONTEXT (field) != record_type)
     /* Check if there is a field with name COMPONENT in the record.  */
     {
-      if (component == 0)
-	gigi_abort (511);
-
-      /* ??? Explore later if we can use the TYPE_LANG_SPECIFIC optimization
-	 that appears in C version of this function.  */
+      tree new_field;
 
       /* First loop thru normal components.  */
 
-      for (field = TYPE_FIELDS (record_type); field;
-	   field = TREE_CHAIN (field))
-	if (DECL_NAME (field) == component)
+      for (new_field = TYPE_FIELDS (record_type); new_field != 0;
+	   new_field = TREE_CHAIN (new_field))
+	if (DECL_ORIGINAL_FIELD (new_field) == field
+	    || new_field == DECL_ORIGINAL_FIELD (field)
+	    || (DECL_ORIGINAL_FIELD (field) != 0
+		&& (DECL_ORIGINAL_FIELD (field)
+		    == DECL_ORIGINAL_FIELD (new_field))))
 	  break;
 
       /* Next, loop thru DECL_INTERNAL_P components if we haven't found
@@ -1258,20 +1513,21 @@ build_simple_component_ref (record_variable, component, field)
          is required to avoiding hidden homonymous fields in the
          _Parent field.  */
 
-      if (field == 0)
-	for (field = TYPE_FIELDS (record_type); field;
-	     field = TREE_CHAIN (field))
-	  if (DECL_INTERNAL_P (field))
+      if (new_field == 0)
+	for (new_field = TYPE_FIELDS (record_type); new_field != 0;
+	     new_field = TREE_CHAIN (new_field))
+	  if (DECL_INTERNAL_P (new_field))
 	    {
 	      tree field_ref
 		= build_simple_component_ref (record_variable, 
-					      NULL_TREE, field);
-	      ref = build_simple_component_ref (field_ref, 
-						component, NULL_TREE);
+					      NULL_TREE, new_field);
+	      ref = build_simple_component_ref (field_ref, NULL_TREE, field);
 
 	      if (ref != 0)
 		return ref;
 	    }
+
+      field = new_field;
     }
 
   if (field == 0)
@@ -1309,8 +1565,7 @@ build_component_ref (record_variable, component, field)
 
   else if (field != 0)
     return build1 (NULL_EXPR, TREE_TYPE (field),
-		   build_call_0_expr (raise_constraint_error_decl));
-
+		   build_call_raise (raise_constraint_error_decl));
   else
     gigi_abort (512);
 }
@@ -1350,9 +1605,9 @@ build_call_alloc_dealloc (gnu_obj, gnu_size, align, gnat_proc, gnat_pool)
 	Entity_Id gnat_size_type
 	  = Etype (Next_Formal (Next_Formal (First_Formal (gnat_proc))));
 	tree gnu_size_type = gnat_to_gnu_type (gnat_size_type);
-	tree gnu_proc = gnat_to_gnu_entity (gnat_proc, NULL_TREE, 0);
+	tree gnu_proc = gnat_to_gnu (gnat_proc);
 	tree gnu_proc_addr = build_unary_op (ADDR_EXPR, NULL_TREE, gnu_proc);
-	tree gnu_pool = gnat_to_gnu_entity (gnat_pool, NULL_TREE, 0);
+	tree gnu_pool = gnat_to_gnu (gnat_pool);
 	tree gnu_pool_addr = build_unary_op (ADDR_EXPR, NULL_TREE, gnu_pool);
 	tree gnu_args = NULL_TREE;
 	tree gnu_call;
@@ -1390,9 +1645,9 @@ build_call_alloc_dealloc (gnu_obj, gnu_size, align, gnat_proc, gnat_pool)
 	Entity_Id gnat_size_type 
 	  = Etype (Next_Formal (First_Formal (gnat_proc)));
 	tree gnu_size_type = gnat_to_gnu_type (gnat_size_type);
-	tree gnu_proc = gnat_to_gnu_entity (gnat_proc, NULL_TREE, 0);
+	tree gnu_proc = gnat_to_gnu (gnat_proc);
 	tree gnu_proc_addr = build_unary_op (ADDR_EXPR, NULL_TREE, gnu_proc);
-	tree gnu_pool = gnat_to_gnu_entity (gnat_pool, NULL_TREE, 0);
+	tree gnu_pool = gnat_to_gnu (gnat_pool);
 	tree gnu_pool_addr = build_unary_op (ADDR_EXPR, NULL_TREE, gnu_pool);
 	tree gnu_args = NULL_TREE;
 	tree gnu_call;
@@ -1441,10 +1696,14 @@ build_allocator (type, init, result_type, gnat_proc, gnat_pool)
   tree ptr_type;
   tree result;
 
+  /* If the initializer, if present, is a NULL_EXPR, just return a new one.  */
+  if (init != 0 && TREE_CODE (init) == NULL_EXPR)
+    return build1 (NULL_EXPR, result_type, TREE_OPERAND (init, 0));
+
   /* If RESULT_TYPE is a fat or thin pointer, set SIZE to be the sum of the
      sizes of the object and its template.  Allocate the whole thing and
      fill in the parts that are known.  */
-  if (TYPE_FAT_OR_THIN_POINTER_P (result_type))
+  else if (TYPE_FAT_OR_THIN_POINTER_P (result_type))
     {
       tree template_type
 	= (TYPE_FAT_POINTER_P (result_type)
@@ -1532,35 +1791,35 @@ build_allocator (type, init, result_type, gnat_proc, gnat_pool)
 	size = build (WITH_RECORD_EXPR, sizetype, size, init);
     }
 
-  /* If we are at top-level and SIZE is a constant, we can actually
-     allocate an object of TYPE and point to it.
-     ??? At some point, we should make an attempt to do this above.  */
-  if (global_bindings_p () && TREE_CODE (size) == INTEGER_CST
-      && ! TREE_OVERFLOW (size) && TREE_INT_CST_HIGH (size) == 0
-      && TREE_INT_CST_LOW (size) < 200000
-      && 0)
-    {
-      char name[20];
+  /* If the size overflows, pass -1 so the allocator will raise
+     storage error.  */
+  if (TREE_CODE (size) == INTEGER_CST && TREE_OVERFLOW (size))
+    size = convert (sizetype, build_int_2 (-BITS_PER_UNIT, ~0));
 
-      type = maybe_pad_type (type, size, 0, Empty, NULL_PTR, 0);
-      sprintf (name, "__V%d", alloc_var_index++);
-      result = create_var_decl (get_identifier (name), NULL_TREE, type, init,
-				0, 0, 0, 1, NULL_PTR);
-      result = build_unary_op (ADDR_EXPR, NULL_TREE, result);
-      init = 0;
+  /* If this is a type whose alignment is larger than the
+     biggest we support in normal alignment and this is in
+     the default storage pool, make an "aligning type", allocate
+     it, point to the field we need, and return that.  */
+  if (TYPE_ALIGN (type) > BIGGEST_ALIGNMENT
+      && No (gnat_proc))
+    {
+      tree new_type = make_aligning_type (type, TYPE_ALIGN (type), size);
+
+      result = build_call_alloc_dealloc (NULL_TREE, TYPE_SIZE (new_type),
+					 BIGGEST_ALIGNMENT, Empty, Empty);
+      result = save_expr (result);
+      result = convert (build_pointer_type (new_type), result);
+      result = build_unary_op (INDIRECT_REF, NULL_TREE, result);
+      result = build_component_ref (result, NULL_TREE,
+				    TYPE_FIELDS (new_type));
+      result = convert (result_type,
+			build_unary_op (ADDR_EXPR, NULL_TREE, result));
     }
   else
-    {
-      /* If the size overflows, pass -1 so the allocator will raise
-	 storage error.  */
-      if (TREE_CODE (size) == INTEGER_CST && TREE_OVERFLOW (size))
-	size = convert (sizetype, build_int_2 (-BITS_PER_UNIT, ~0));
-
-      result = convert (result_type,
-			build_call_alloc_dealloc (NULL_TREE, size,
-						  TYPE_ALIGN (type),
-						  gnat_proc, gnat_pool));
-    }
+    result = convert (result_type,
+		      build_call_alloc_dealloc (NULL_TREE, size,
+						TYPE_ALIGN (type),
+						gnat_proc, gnat_pool));
 
   /* If we have an initial value, put the new address into a SAVE_EXPR, assign
      the value, and return the address.  Do this with a COMPOUND_EXPR.  */
@@ -1594,6 +1853,7 @@ fill_vms_descriptor (expr, gnat_formal)
   tree const_list = 0;
 
   expr = maybe_unconstrained_array (expr);
+  mark_addressable (expr);
 
   for (field = TYPE_FIELDS (record_type); field; field = TREE_CHAIN (field))
     {
